#!/usr/local/bin/perl
#
# A CGI which Builds a big tar file based on various parameters passed via
# the HTML POST method. This script uses the `CGI' perl package; see
# `http://www-genome.wi.mit.edu/ftp/pub/software/WWW/cgi_docs.html' for
# documentation. 
#
# jhrg 8/2/96

# $Id: tar-builder.pl,v 1.68 1999/07/26 23:11:13 edavis Exp $

use CGI;
$query = new CGI;

# Part names in the CGI parameters translate to these files.
%source_part = ('core', "DODS-dap-3.0.3.tar.gz",
		'ncdods', "DODS-nc-dods-3.0.3.tar.gz",
		'jgdods', "DODS-jg-dods-3.0.3.tar.gz",
		'hdf', "DODS-hdf-dods-3.0.3.tar.gz",
		'matlab', "DODS-math-dods-3.0.3.tar.gz",
		'dsp', "DODS-dsp-dods-3.0.2.tar.gz",
		'writeval', "DODS-writeval-3.0.3.tar.gz",
		'matlabgui', "DODS-matlab-GUI-3.0.3.tar.gz",
#		'docs', "DODS-doc-2.15.tar.gz",
		'packages', "DODS-packages-2.17.tar.gz",
		'ff', "DODS-ff-dods-3.0.3.tar.gz");

$clients = "netcdf jgofs matlab client_core";
$servers = "nc jg dsp hdf mat ff";

$etcdir = "DODS/etc/";
$bindir = "DODS/bin/";
$libdir = "DODS/lib/";
$packlibdir = "DODS/packages/lib/";
$packbindir = "DODS/packages/bin/";

# Note: in the following assoc array, `client_core' and `server_core' hold
# files that are bundled with every binary tar file that contains any clients
# or servers respectively.

%binary_part = ('netcdf', "${bindir}dncdump ${bindir}dncview",
		'jgofs', "${bindir}list",
		'matlab', "${bindir}matlab-gui",
		'client_core', "${etcdir}*.tcl ${etcdir}INSTALL-clients ${bindir}geturl ${bindir}urlqueue ${bindir}loaddods.* ${bindir}whodods.m ${bindir}writeval",

		'nc', "${etcdir}nph-nc ${etcdir}nc_das ${etcdir}nc_dds ${etcdir}nc_dods",

		'jg', "${etcdir}nph-jg ${etcdir}jg_das ${etcdir}jg_dds ${etcdir}jg_dods ${etcdir}def ${etcdir}.objects",

		'dsp', "${etcdir}nph-dsp ${etcdir}dsp_das ${etcdir}dsp_dds ${etcdir}dsp_dods",

		'hdf', "${etcdir}nph-hdf ${etcdir}hdf_das ${etcdir}hdf_dds ${etcdir}hdf_dods",

		'mat', "${etcdir}nph-mat ${etcdir}mat_das ${etcdir}mat_dds ${etcdir}mat_dods",

		'ff', "${etcdir}nph-ff ${etcdir}ff_das ${etcdir}ff_dds ${etcdir}ff_dods ${bindir}chkform ${bindir}newform ${bindir}bufform ${bindir}readfile",

		'server_extras', "${etcdir}usage ${etcdir}asciival ${etcdir}*.pm ${etcdir}*.tcl ${etcdir}INSTALL-servers ${etcdir}deflate",

		'libdap', "${libdir}libdap++.a DODS/include/*",
		'libnc-dods', "${libdir}libnc-dods.a",
		'libjg-dods', "${libdir}libjg-dods.a",

		'support', "${packlibdir}libwww.a ${packlibdir}libtcl*.a ${packlibdir}libexpect*.a ${packlibdir}libz.a ${packbindir}wish*");

# Where does GNU tar live?
#--URI--$gnu_tar = "/usr/local/bin/tar";
$gnu_tar = "/local/gnu/bin/gtar";

# ftp_dir is the directory where the tar files reside. The custom tar file
# will reside in $ftp_dir/custom. This directory should be `cleaned'
# periodically using "rm -f `find . -ctime +24 -print`" to remove all files
# older than 24 hours.
#--URI-- $ftp_dir = "/data1/ftp/pub/dods";
$ftp_dir = "/home/ftp/pub/dods";

# html_page is a page of html which contains directions about expanding the
# tar file, building gzip if need be and generally compiling DODS.
# $html_source_page = "/usr/local/apache/share/cgi-bin/ftp_dods_source.html";
# $html_binary_page = "/usr/local/apache/share/cgi-bin/ftp_dods_binary.html";
$html_source_page = "ftp_dods_source.html";
$html_binary_page = "ftp_dods_binary.html";

# Log file used to record downloads. This contains all of the params passed
# to this CGI along with the time and request address. The access log lists
# of httpd lists the machine IP, time and GET request invoking tar-builder,
# but not the params sent to tar builder. This second log file tells *what*
# users wanted when they used tar-builder. 12/24/98 jhrg

#--URI-- $log = "/usr/local/apache/var/log/tar-builder-log";
$log = "/dist/ns-home/httpd-80/logs/tar-builder-log";

# End of global variables. 

############################################################################

# Print an HTML document with an error message in it (along with a link back
# to the previous page.
#
# Returns: never returns

sub error_html {
    my($message) = @_;

    print $query->header;
    print "\n\n";
    print "<html><title>Archive error</title>\n";
    print "<body>\n";

    print 'Archive error: ';
    print $message;
    print "<p><a href=\"mailto:support\@unidata.ucar.edu\">Report this error</a>";
    print "<p>Return to the <a href=\"" . $query->referer . "\">Current release</a> form.\n";

    print "</body></html>\n";

    &write_log_entry("Error");

    exit;
}

# Look at the comptool parameter; if it was set return the corresponding GNU
# tar compression switch and file name extension.

sub comp_tool {
    # Type of compression to use.
    $comptool = $query->param('comptool');

    if ($comptool eq 'gzip') {
	# $extension = 'gz';
#--URI--	return ('gzip', '/usr/local/bin/gzip', '.gz');
	return ('gzip', '/local/gnu/bin/gzip', '.gz');
    }
    elsif ($comptool eq 'compress') {
	# $extension = 'Z';
#--URI--	return ('compress', '/usr/ucb/compress', '.Z');
	return ('compress', '/usr/bin/compress', '.Z');
    }
    else {
	return ('', '','');
    }
}

# Write the tar `file' created by build_source_archive or build_binary_archive
# to a new file in the custom subdir of the ftp directory. 
# NB: This function takes one parameter; the file handle of the tar file
# pipe.
#
# Returns: The name of the newly created tar file, without the
# `${ftp_dir}/custom' part. 

sub write_archive {
    my($TAR) = @_;

    ($dummy, $dummy, $comp) = &comp_tool; # Only used for the extension.

    $file = "DODS-$$.tar" . $comp;

    open(FILE, "> ${ftp_dir}/custom/${file}") || &error_html("Could not create the custom tar file!\n");
    print FILE <$TAR>;

    close $TAR;
    close FILE;

    return $file;
}

# Given the name of a file which contains the custom tar archive and an HTML
# page which tells users how to download that tar file, perform various
# substitutions on the tar file and write the result to stdout.
#
# Returns: The status of the close operation on the HTML template.

sub send_page {
    my($file, $html_page) = @_;

#--URI--    $file_url = "ftp://dods.gso.uri.edu/pub/dods/custom/$file";
    $file_url = "ftp://ftp.unidata.ucar.edu/pub/dods/custom/$file";
    @file_size = stat("${ftp_dir}/custom/${file}");
    $file_size = int($file_size[7]/1024) . "K";
    $referer = $query->referer;
    $platform = $query->param('platform');
#--URI--    $gnu_gzip = "ftp://dods.gso.uri.edu/pub/dods/binaries/${platform}/support/gzip";
    $gnu_gzip = "ftp://ftp.unidata.ucar.edu/pub/dods/binaries/${platform}/support/gzip";
    ($comptool, $dummy, $dummy) = &comp_tool;

    open(HTML, "< $html_page") || &error_html("Could not open the HTML template!\n");

    print $query->header;

    while (<HTML>) {
	s@#tar-file#@${file}@g;
	s@#tar-file-url#@${file_url}@g;
	s@#tar-file-size#@${file_size}@g;
	s@#referer#@${referer}@g;
	s@#gnu-gzip#@${gnu_gzip}@g;
	s@#comptool#@${comptool}@g;

	print;
    }

    close HTML;
}

sub build_source_archive {
    # components is a list of all the name=value pairs sent from the form.
    @components = $query->param;

    # How should each of the components should be included? 
    foreach $comp (@components) {
	$params .= $source_part{$comp} . " "; # source_part is defined above.
    }
    
    chdir "${ftp_dir}" || &error_html("Could not get to the archive!\n");

    open(TAR, "$gnu_tar --exclude '*/CVS' --create --file - $params |") || &error_html("Could not open tar!\n");
    return TAR;
}

sub build_binary_archive {
    @components = $query->param;

    $client_found = 0;
    $server_found = 0;

    foreach $comp (@components) {
	$params .= $binary_part{$comp} . " ";
	if (!$client_found && $clients =~ /$comp/) {
	    $client_found = 1;
	    $params .= $binary_part{'client_core'} . " ";
	}
	if (!$server_found && $servers =~ /$comp/) {
	    $server_found = 1;
	    $params .= $binary_part{'server_extras'} . " ";
	}
    }

    if ($client_found || $server_found) {
	$params .= "DODS/README-binary ";
    }

    $platform = $query->param('platform');
    if ($platform eq '') {
	&error_html("Could not figure out which platform you want binaries for!\n");
    }

    chdir "${ftp_dir}/binaries/${platform}" || &error_html("Could not get to archive!\n");

    #print "params: ", $params, "\n";
    #print "pwd: ", `pwd`, "\n";

    ($dummy, $comp, $dummy) = &comp_tool;

    # Removed --dereference switch. This means that symbolic links in the
    # binary distributions will be preserved (and not replaced by the files
    # they reference). 3/8/99 jhrg
    # 
    # Added --dereference back in. This is necessary because the ML GUI's 
    # DATASETS directory is softlinked. 3/8/99 jhrg
    open(TAR, "$gnu_tar --exclude '*/CVS' --dereference --create --file - $params | $comp -c |");
    return TAR;
}    

# Write a log file entry. This logfile should be separate from the access log
# file used by the host site's httpd to simplify looking at download stats.

sub write_log_entry {
    my ($status) = @_;

    open(LOG, ">> $log") || die "Could not open the log file, stopped";

    # Print `IP [date and time] params status'
    my $ip = $ENV{"REMOTE_ADDR"};
    my $time = scalar localtime;
    print LOG "$ip [$time] ";

    my @components = $query->param;
    foreach $comp (@components) {
	if ($comp eq "tar-type" || $comp eq "platform") {
	    my $value = $query->param("$comp");
	    print LOG "$value ";
	}
	else {
	    print LOG "$comp ";
	}
    }
    print LOG "$status\n";

    close(LOG);
}

# Main starts here.

# $directions: This parameter contains the name of an HTML page of directions
# sent back to the user. If it is not present, use the default source or
# binary pages. This makes it easier to send special directions in response
# to the custom download page(s) for the GUI(s).

my $directions = $query->param('directions-page');

umask( 022);
if ($query->param('tar-type') eq 'source') {
    $TAR = &build_source_archive;
    $file = &write_archive($TAR);
    if ($directions eq '') {
	$directions = $html_source_page;
    }
#--URI--    $directions = "/usr/local/apache/share/cgi-bin/" . $directions;
    $directions = "/dist/cgi-bin/dods/tar-builder/" . $directions;
    &send_page($file, $directions);
    &write_log_entry("OK")
} elsif ($query->param('tar-type') eq 'binary') {
    $TAR = &build_binary_archive;
    $file = &write_archive($TAR);
    if ($directions eq '') {
	$directions = $html_binary_page;
    }
#--URI--    $directions = "/usr/local/apache/share/cgi-bin/" . $directions;
    $directions = "/dist/cgi-bin/dods/tar-builder/" . $directions;
    &send_page($file, $directions);
    &write_log_entry("OK")
} else {
    &error_html("Could not figure out if you wanted binary or source code!\n");
}
